"""Command-line interface"""

import re, sys

from .colorls import cols_format


LINE_REGEX = re.compile(r"\s*\S+")


def main(args):
    """Called indirectly from a `c*ls` script.  Colorise the input as required.

    Parameters: $size_column $inums $ownership_cols [ $human ]"""

    size_column = int(args[1]) - 1
    inums_on = args[2] == "y"
    ownership_cols = int(args[3])
    if inums_on:
        dir_column = 1
    else:
        dir_column = 0
    try:
        human = args[4] == "y"
    except IndexError:
        human = False

    ## print(len(args))
    for line in sys.stdin:
        cols = LINE_REGEX.findall(line)
        # TO-DO: better heuristics
        if line and line[-1] != ":" and len(cols) >= size_column + 3:
            print(cols_format(cols, ownership_cols, size_column, dir_column))
        else:
            # Pass dir header lines and block counts through unchanged
            print(line, end='')
    return 0


# *** MAINLINE ***
if __name__ == '__main__':
    try:
        sys.exit(main(sys.argv))
    except (BrokenPipeError, IOError):
        # Avoid "Exception ignored ..." and "BrokenPipeError: [Errno 32]..."
        # messages from the Python interpreter
        sys.stderr.close()



"""Top-level package for c*ls: ls wrapper."""

__author__ = """Alastair Irvine"""
__email__ = 'alastair@plug.org.au'
__version__ = '1.2.0'



# *** DEFINITIONS ***
size_col_width = 12
ownership_col_width = 11



"""Main module."""

import re
from typing import Dict, Tuple, List, Union, AnyStr, Iterable, Callable, Type, Optional

import colored

from . import size_col_width, ownership_col_width

## MULTI_COL_WIDTH = 20 + SIZE_COL_WIDTH

ISDIR_REGEX = re.compile(r"\s*d")


# == Interface functions ==
def cols_format(cols: int, ownership_cols: int, size_column: int, dir_column: int) -> str:
    size_str = cols[size_column].strip()
    # Pad based on the actual width, not the number of characters (as they
    # include zero-width highlighting data)
    padding = " " * (size_col_width - len(size_str))
    if ISDIR_REGEX.match(cols[dir_column]):
        # Don't highlight directory sizes
        size_output = size_str
    else:
        size_output = colorise(size_str)
    if ownership_cols == 0:
        # no owner/group columns
        return "%s %s%s" % ("".join(cols[0:size_column]), padding + size_output,
                             "".join(cols[size_column+1:]))
    else:
        # at least one owner/group column
        ownership_output = ownership_tweak(cols[size_column-ownership_cols:size_column])
        ## sep_str = " " * (MULTI_COL_WIDTH - len(ownership_output + size_str))
        multicol_output = ownership_output + " " + padding + size_output
        return "%s%s%s" % ("".join(cols[0:size_column-ownership_cols]),
                           multicol_output, "".join(cols[size_column+1:]))


# == Internal functions ==
# Recursive function to highlight every second group of 3 digts (starting from
# the right)
def tweak(s: Optional[str], n=0) -> Tuple[int, str]:
    if not s:
        return 0, ""
    else:
        count, s_prime = tweak(s[1:], n+1)
        # Odd modulo 3
        if int(count / 3) % 2 == 1 and s_prime:
            return count + 1, s[0] + "\b_" + s_prime
        else:
            return count + 1, s[0] + s_prime


def colorise(s: str) -> str:
    """Colorise integer values using underlines and colorise values with a suffix
    using various colours."""

    COLOURS = {"E": "purple_1b", "P": "deep_pink_2", "G": "magenta_2b", "M": "indian_red_1c", "K": "tan"}
    try:
        # See if the size is an integer values
        int(s) 

        ## return s + "\b_"
        length, string = tweak(s)
        return string
    except ValueError:
        # Size is a string, e.g. "3.5M"
        suffix = s[-1]
        try:
            colour = COLOURS[suffix]
            return "%s%s%s%s" % (colored.fg(colour), s[0:-1], colored.attr("reset"), suffix)
        except KeyError:
            return s


def ownership_tweak(cols: Iterable[str]) -> str:
    """Pad (and truncate if necessary) username/groupname columns."""

    def fragment(s: str) -> str:
        """Return a username/groupname truncated as per ps(1) if it won't fit."""
        if len(s) >= ownership_col_width:
            return s[0:ownership_col_width-2] + "+"
        else:
            # Left-justified in a field of one less character than the total column width
            return "%-*s" % (ownership_col_width-1, s)

    # The first column has one space of pre-padding
    return " " + "".join(fragment(col.strip()) + " " for col in cols)


# *** MODULE MAINLINE ***
colored.set_tty_aware(False)



#!/usr/bin/env python

"""The setup script."""

import os.path
from setuptools import setup, find_packages
from setuptools.command.install_scripts import install_scripts
from setuptools.command.develop import develop

from pipenv.project import Project
try:
    from pipenv.utils.dependencies import convert_deps_to_pip
except ImportError:
    from pipenv.utils import convert_deps_to_pip


pfile = Project(chdir=False).parsed_pipfile
requirements = convert_deps_to_pip(pfile['packages'], r=False)
test_requirements = convert_deps_to_pip(pfile['dev-packages'], r=False)

hardlink_names = ("clls", "cllls", "cals", "calls", "callls", "csls", "cslls", "csllls", "csals", "csalls", "csallls")


class CustomInstallScriptsCommand(install_scripts):
    """Customized setuptools install_scripts command - installs hard links to 'cls'."""
    def run(self):
        ## print("Hello, developer, how are you? :)")
        ## bs_cmd = self.get_finalized_command('build_scripts')
        inst_cmd = self.get_finalized_command('install_scripts')
        install_scripts.run(self)
        ## print(bs_cmd.__dict__)
        ## print(inst_cmd.__dict__)

        #post-processing code
        for name in hardlink_names:
            target = os.path.join(inst_cmd.install_dir, name)
            ## self.copy_file(os.path.join(bs_cmd.build_dir, "cls"), target)
            self.copy_file(os.path.join(inst_cmd.install_dir, "cls"), target, link='hard')
            self.outfiles.append(target)


class CustomDevelopCommand(develop):
    """Customized setuptools develop command - installs hard links to 'cls'."""
    def run(self):
        ## print("Hello, developer, how are you? :)")
        inst_cmd = self.get_finalized_command('develop')
        develop.run(self)
        ## print(inst_cmd.__dict__)

        #post-processing code
        for name in hardlink_names:
            target = os.path.join(inst_cmd.script_dir, name)
            self.copy_file(os.path.join(inst_cmd.script_dir, "cls"), target, link='hard')
            ## self.outfiles.append(target)


# *** MAINLINE ***
with open('README.md') as readme_file:
    readme = readme_file.read()

setup_requirements = ['bumpversion']

setup(
    author="Alastair Irvine",
    author_email='alastair@plug.org.au',
    python_requires='>=3.4',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Environment :: Console',
        'Topic :: Utilities',
        'Operating System :: POSIX :: Linux',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Unix Shell',
    ],
    description="c*ls commands that use colour and pipe output to less, with a sensible prompt",
    install_requires=requirements,
    license="GNU General Public License v2",
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='cls, ls, less, colour, color',
    name='colorls',
    packages=find_packages(include=['colorls', 'colorls.*']),
    scripts=['bin/cls'],
    cmdclass={
        'install_scripts': CustomInstallScriptsCommand,
        'develop': CustomDevelopCommand,
    },
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/unixnut/cls',
    version='1.2.0',
    zip_safe=False,
)
